
       //=======================================================//    _\|/_
      //  __  _____           ___                    ___       //      /|\
     //  /      |      ^     |   \  |         ^     |   \     //          _\|/_
    //   \__    |     / \    |___/  |        / \    |___/    //            /|\
   //       \   |    /___\   |  \   |       /___\   |   \   // _\|/_
  //     ___/   |   /     \  |   \  |____  /     \  |___/  //   /|\     
 //                                                       //            _\|/_
//=======================================================//              /|\

/*
 *  hdyn.h: derived class for nbody systems, when using a Hermite integrator
 *.............................................................................
 *    version 1:  Dec 1992   Piet Hut, Steve McMillan, Jun Makino
 *    version 2:
 *.............................................................................
 *     This file includes:
 *  1) definition of class hdyn
 *.............................................................................
 */

#ifndef  STARLAB_HDYN_H
#  define  STARLAB_HDYN_H

#include  "dyn.h"
//#include  "hdyn_kepler.h"   NOTE: this file is included at the end instead,
//                            because it needs to know about the hdyn class

/*-----------------------------------------------------------------------------
 *  hdyn  --  a derived class of dynamical particles, with enough information
 *             to integrate the equations of motions with a 4th-order Hermite
 *             polynomial integrator.
 *-----------------------------------------------------------------------------
 */
class  hdyn : public dyn
{
    protected:
    
	real  time;             //
	real  timestep;         //
	real  pot;              //  potential
	vector  jerk;           //  (d/dt) acc
	vector  old_acc;        //
	vector  old_jerk;       //
	vector  pred_pos;       //
	vector  pred_vel;       //
// variables for unperturbed kepler motion
	real  perturbation_squared;  // relative perturbation squared
	real  unperturbed_timestep;  // timestep for the unperturbed motion
// end of kepler motion
	real d_nn_sq;           // distance squared to the nearest neighbor
	hdyn* nn;               // pointer to the nearest neighbor
	int  n_perturbers;	// number of perturbers
	hdyn** perturber_list;  // pointer to perturber array
	real  perturbation_radius_squared;

// variables for HARP-3
        int harp3_index;     // address of this particle on harp3 memory
    public:

        hdyn(hbpfp the_hbpfp = new_hydrobase, sbpfp the_sbpfp = new_starbase)
	   : dyn(the_hbpfp, the_sbpfp)
	    {
	    nn = NULL;
	    perturber_list = NULL;
	    n_perturbers = 0;
	    time = timestep = pot = d_nn_sq = 0;
            jerk = old_acc = old_jerk = pred_pos = pred_vel = 0;
	    }
 
	void  set_timestep(real dt)       {timestep = dt;}
	real  get_timestep()              {return timestep;}

	real  get_time()                  {return time;}
	void  set_time(real t)            {time = t;}

	void  clear_interaction()         {acc=jerk=pot = 0;}
	
	void  set_acc_and_jerk_and_pot(const vector & a,
				       const vector & j, real p)
	   {acc = a; jerk = j; pot = p;}
#if 0
	real  get_next_time()             {return time + timestep;}
#else
        real  get_next_time();
#endif
	vector  get_pred_pos()            {return pred_pos;}

	vector  get_pred_vel()            {return pred_vel;}

	real  get_pot()                   {return pot;}
	void  set_pot(real p)		  {pot = p;}
	void  clear_pot()                 {pot = 0;}
	void  inc_pot(const real& d_pot)  {pot += d_pot;}

        void  set_jerk(const vector& new_jerk)     {jerk = new_jerk;}
        void  set_old_acc(const vector& new_acc)     {old_acc = new_acc;}
        void  set_old_jerk(const vector& new_jerk)     {old_jerk = new_jerk;}
	void  clear_jerk()                         {jerk = 0;}
	void  inc_jerk(const vector& d_jerk)       {jerk += d_jerk; }
	void  inc_old_acc(const vector& d_acc)       {old_acc += d_acc; }
	void  inc_old_jerk(const vector& d_jerk)       {old_jerk += d_jerk; }
	vector  get_jerk()                         {return jerk;}
	vector  get_old_acc()                         {return old_acc;}
	vector  get_old_jerk()                         {return old_jerk;}

	hdyn * get_nn(){return nn;}
	void set_nn(hdyn * new_nn){nn = new_nn;}
	real get_d_nn_sq(){return d_nn_sq;}
	void set_d_nn_sq(real d){d_nn_sq = d;}

	int get_harp3_index(){return harp3_index;}
	void set_harp3_index(int hindex){harp3_index = hindex;}

	hdyn * get_parent()
	    {return (hdyn*) node::get_parent();}
	hdyn * get_oldest_daughter()
	    {return (hdyn*)node::get_oldest_daughter();}
	hdyn * get_younger_sister()
	    {return (hdyn*) node::get_younger_sister();}
	hdyn * get_elder_sister()
	    {return (hdyn*) node::get_elder_sister();}

        hdyn * get_root()
            {return (hdyn*) node::get_root();}
        hdyn * get_top_level_node()
            {return (hdyn*) node::get_top_level_node();}
        hdyn * get_binary_sister()
            {return (hdyn*) node::get_binary_sister();}

	virtual  istream& scan_dyn_story(istream&);
	virtual  ostream& print_dyn_story(ostream &);
	
	void  init_pred()                 {pred_pos = pos; pred_vel = vel;}

	void  store_old_force()	          {old_acc = acc; old_jerk = jerk;}
#if 0
	void  predict_loworder(const real t)
	    {
	    real dt = t - time;
	    real dt3 = dt * ONE_THIRD;
	    real dt2 = dt * 0.5;
	    pred_pos = ((jerk*dt3 + acc)*dt2 + vel)*dt + pos;
	    pred_vel = (jerk*dt2 + acc)*dt + vel;
	    }
#else
        void  predict_loworder(const real t); 
#endif
	void  set_first_timestep(real, real);
	void  flat_set_first_timestep(real, real);

	void  correct();
	void  update(const real, const real);
	void  flat_update(const real, const real);
	void  correct_and_update(const real, const real, const real);

	void  flat_accumulate_acc_and_jerk(hdyn &, real);
	void  flat_calculate_acc_and_jerk(hdyn *, real);

        void  integrate_node(hdyn*, real, real, real,
			     real d_min_sq = VERY_LARGE_NUMBER);
        void  calculate_acc_and_jerk(real, real);
        void  calculate_acc_and_jerk_on_top_level_node(real, real);
        void  calculate_acc_and_jerk_on_low_level_node(real);
        void  calculate_partial_acc_and_jerk(hdyn*, hdyn*, real, vector&,
					     vector&, real&, real&, hdyn*&,
					     bool, bool, bool);
        void  calculate_partial_acc_and_jerk_on_leaf(hdyn*, hdyn*, real,
						     vector&, vector&,
						     real&, real&, hdyn*&,
						     bool, bool, bool);
        void  tree_walk_for_partial_acc_and_jerk_on_leaf(hdyn*, hdyn*, vector,
							 vector, real, vector&,
							 vector&, real&, real&,
							 hdyn*&, bool, bool);

	void  setup_binary_node();

        void  synchronize_node(real, real, real, real,
			       real d_min_sq = VERY_LARGE_NUMBER);

        real  distance_to_sister_squared();

        hdyn* new_sister_node(real, bool&);

        bool is_unperturbed_and_approarching(real gamma2);
        void startup_unperturbed_motion();
        void integrate_unperturbed_motion(real gamma2);
        void print_perturber_list(ostream & s = cout);
        void clear_perturber_list(){n_perturbers = 0;}
};

typedef hdyn * hdynptr;  // to enable dynamic array declarations such as
                         //    hdyn** hdyn_ptr_array = new hdynptr[n];
                         // (note that the following expression is illegal:
                         //    hdyn** hdyn_ptr_array = new (hdyn *)[n];)

inline  node * new_hdyn(hbpfp the_hbpfp, sbpfp the_sbpfp)
    {return (node *) new hdyn(the_hbpfp, the_sbpfp);}

inline hdyn * mkhdyn(int n, hbpfp the_hbpfp = new_hydrobase,
	                    sbpfp the_sbpfp = new_starbase)
    {return  (hdyn *) mk_flat_tree(n, new_hdyn, the_hbpfp, the_sbpfp);}

inline  hdyn * get_hdyn(istream & s, hbpfp the_hbpfp = new_hydrobase,
		                     sbpfp the_sbpfp = new_starbase)
    {return  (hdyn *) get_node(s, new_hdyn, the_hbpfp, the_sbpfp);}
    
inline hdyn * common_ancestor(hdyn * bi, hdyn * bj)
    {return (hdyn*) common_ancestor((node*)bi, (node*)bj);}

#define  put_hdyn  put_node

#include  "hdyn_kepler.h"

// Declaration of member defined in hdyn_tt.C

// the second definition below, of hdyn_MF_ptr, should probably be local to
// the file hdyn_tt.C  -- we should look at this later.

typedef vector (hdyn::*hdyn_VMF_ptr)(void);   // vector member function pointer
typedef void (hdyn::*hdyn_MF_ptr)(const vector &);   // member function pointer

    //  NOTE:
    //  all the "hdyn" declarations below should be changed to "node" here
    //  (just as was done with get_node() ).

void check_consistency_of_node(hdyn &, hdyn_VMF_ptr, char *);
void check_consistency_of_nodes(hdyn *);
void pp3(hdyn &, ostream & s = cout, int i = 0);
void pp3_cm(hdyn &, ostream & s = cout, int i = 0);
void move_node(hdyn&, hdyn&);
void move_node_by_index(int, int, hdyn *);
void create_binary_from_toplevel_nodes(hdyn&, hdyn&);

vector something_relative_to_ancestor(hdyn*, hdyn*, hdyn_VMF_ptr);

// This is the same function as something_relative_to_root, but
// g++ versions after 2.7.0 complain about the ambiguity for pos,
// vel, and acc if we name it something_relative_to_root!
// This version is actually needed (and used) only for get_jerk.

vector hdyn_something_relative_to_root(hdyn*, hdyn_VMF_ptr);

void remove_node_and_correct_upto_ancestor(hdyn&, hdyn&);

hdyn* node_to_move(hdyn*, real&);

void get_nodes_to_move(hdyn*, hdyn **, int&, real&);

void initialize_system_phase1(hdyn*, real);
void initialize_system_phase2(hdyn*, hdyn*, real, real);

void predict_loworder_all(hdyn*, real);

void update_binary_sister(hdyn* bi);
#endif

//=======================================================================//
//  +---------------+        _\|/_        +------------------------------\\
//  |  the end of:  |         /|\         |  inc/hdyn.h
//  +---------------+                     +------------------------------//
//========================= STARLAB =====================================\\

