
       //=======================================================//    _\|/_
      //  __  _____           ___                    ___       //      /|\
     //  /      |      ^     |   \  |         ^     |   \     //          _\|/_
    //   \__    |     / \    |___/  |        / \    |___/    //            /|\
   //       \   |    /___\   |  \   |       /___\   |   \   // _\|/_
  //     ___/   |   /     \  |   \  |____  /     \  |___/  //   /|\     
 //                                                       //            _\|/_
//=======================================================//              /|\

// kira_harp_include.C:  All conditional code that depends on HARP
//			 availability.  Not compiled into a library,
//			 but instead included directly into kira and
//			 the hdyn version of sys_stats.
//
// STARLAB_HAS_HARP is true only during the building of kira_harp3.
//
// Functions defined:
//
//	bool kira_use_harp
//	void kira_set_harp_cpu
//	void calculate_energies_with_tidal
//	void kira_calculate_top_level_acc_and_jerk
//	void compute_densities

#if defined(STARLAB_HAS_HARP)

    #define HARP_CHECK_COUNT 15000
    #define HARP_MAX_CPU 15

    // Don't make these static -- they are declared extern in hdyn_harp3.C,
    // and will be included directly into the main program of kira_harp3
    // when needed.

    int  harp_check_count = HARP_CHECK_COUNT;
    real harp_max_cpu = HARP_MAX_CPU;

#endif

bool kira_use_harp()
{
#if defined(STARLAB_HAS_HARP)
    return true;
#else
    return false;
#endif
}

void kira_set_harp_cpu(real cpu)
{
#if defined(STARLAB_HAS_HARP)
    harp_max_cpu = cpu;
    harp_check_count = (int)harp_max_cpu * 1000;
#endif
}

void calculate_internal_energies(hdyn* b,
				 real& epot, real& ekin, real& etot,
				 bool cm,		// default = false
				 bool use_grape)	// default = true
{
    // Compute the total internal energy; also compute the "pot" class data.

    // Notes from Steve (8/99):
    //
    //	- calculate_energies_by_harp3 recomputes hdyn::pot, but does
    //	  *not* include the tidal terms.
    //
    //  - new code uses the hdyn version of calculate_energies() (see
    //    ../util/hdyn_tt.C), which sets the hdyn::pot member data,
    //    and also omits the tidal terms.

#if defined(STARLAB_HAS_HARP)

    if (use_grape)
	calculate_energies_by_harp3(b, epot, ekin, etot);
    else
	calculate_energies(b, b->get_eps2(), epot, ekin, etot, cm);

#else

    calculate_energies(b, b->get_eps2(), epot, ekin, etot, cm);

#endif

}

void kira_calculate_top_level_acc_and_jerk(hdyn ** next_nodes,
					   int n_next,
					   xreal time,
					   bool & restart_harp)
{
#if defined(STARLAB_HAS_HARP)

//     if (next_nodes[0]->get_system_time()
// 	>= xreal(2296, 3651856000000000000)) {
//
// 	cerr.precision(HIGH_PRECISION);
// 	cerr << endl << "in kira_calculate_top_level_acc_and_jerk:"
// 	     << endl;
// 	PRC(time); cerr << "  "; xprint(time);
//     }

    harp_calculate_acc_and_jerk(next_nodes, n_next,
				time, restart_harp);
    restart_harp = false;

#else

    for (int i = 0; i < n_next; i++) {
	hdyn *bi = next_nodes[i];
	if (bi->is_top_level_node())
	    bi->top_level_node_real_force_calculation();
    }

#endif
}

void compute_densities(hdyn* b, vector& cod_pos, vector& cod_vel)
{
#if defined(STARLAB_HAS_HARP)
    cerr << "Computing densities using HARP..." << endl;
    real cpu0 = cpu_time();

    harp_calculate_densities(b, 1.0);	// place densities in dyn stories
    real cpu1 = cpu_time();

    compute_mean_cod(b, cod_pos, cod_vel);
    real cpu2 = cpu_time();

    cerr << "CPU times:  density " << cpu1 - cpu0
	 << "  cod " << cpu2 - cpu1
	 << endl;
#endif
}
