
       //=======================================================//    _\|/_
      //  __  _____           ___                    ___       //      /|\
     //  /      |      ^     |   \  |         ^     |   \     //          _\|/_
    //   \__    |     / \    |___/  |        / \    |___/    //            /|\
   //       \   |    /___\   |  \   |       /___\   |   \   // _\|/_
  //     ___/   |   /     \  |   \  |____  /     \  |___/  //   /|\     
 //                                                       //            _\|/_
//=======================================================//              /|\

// hdyn_io:  Starlab hdyn-specific I/O functions.

#include "hdyn.h"
#include "util_io.h"

#ifndef TOOLBOX

// Initialize all static hdyn data here (?)

kira_counters *hdyn::kc		= NULL;    
kira_options *hdyn::options	= NULL;    
kira_diag *hdyn::diag		= NULL;    

int  hdyn::tidal_type	= 0;
real hdyn::omega	= 0;
real hdyn::omega_sq	= 0;
real hdyn::alpha1	= 0;
real hdyn::alpha3	= 0;

bool hdyn::use_dstar	= false;

real hdyn::stellar_encounter_criterion_sq = 1;
real hdyn::stellar_merger_criterion_sq = 1;
real hdyn::stellar_capture_criterion_sq = 1;

hdyn** hdyn::perturbed_list = NULL;
int  hdyn::n_perturbed	= 0;

real hdyn::eta		= 0;
real hdyn::eps		= 0;
real hdyn::eps2		= 0;

real hdyn::d_min_sq	= 0;
real hdyn::lag_factor	= 0;
real hdyn::mbar		= 0;

real hdyn::gamma2	= 0;    
real hdyn::gamma23	= VERY_LARGE_NUMBER;   

real hdyn::initial_step_limit	= 0;
real hdyn::step_limit		= 0;
real hdyn::unpert_step_limit	= 1;

real hdyn::scaled_stripping_radius = -1;

int hdyn::max_slow_factor = 1;			// default is to suppress slow
						// motion, for now
real hdyn::max_slow_perturbation = 1.e-4;	// should be tied to parameters
						// for unperturbed motion
real hdyn::max_slow_perturbation_sq = 1.e-8;

void check_sanity_of_timestep(xreal & time, real & timestep)
{
    if (timestep > 0) {

	// Sanity check for timestep...

	if (fmod(time,  timestep) != 0.0) {

	    cerr << " impossible timestep error \n";

	    cerr.precision(HIGH_PRECISION);
	    PRL(time);
	    PRL(timestep);
	    PRL(1.0/timestep);
	    PRL(fmod(time,  timestep));
	    cerr << " Perform correction... \n";
      
	    real corrected_timestep = 1.0;
	    while (corrected_timestep > timestep*1.1)
		corrected_timestep *= 0.5;

	    timestep = corrected_timestep;
	    real  err;
	    if ((err = fmod(time,  timestep)) != 0.0) {
		time = time - err;
		if (err > 0.5*timestep)
		    time += timestep;
	    }
	    PRL(time);
	    PRL(timestep);
	}
    }
}
    
static bool read_xreal = false;

istream & hdyn::scan_dyn_story(istream & s)
{
    char input_line[MAX_INPUT_LINE_LENGTH];
    real last_real = false;

    while (get_line(s, input_line), strcmp(")Dynamics", input_line)) {

	char keyword[MAX_INPUT_LINE_LENGTH];
	char should_be_equal_sign[MAX_INPUT_LINE_LENGTH];

	sscanf(input_line, "%s%s", keyword, should_be_equal_sign);
	if (strcmp("=", should_be_equal_sign)) {
	    cerr << "Expected '=', but got '" << should_be_equal_sign
		 << endl;
	    exit(1);
	}

	// See xreal notes in dyn_io.C...

    	if (!strcmp("real_system_time", keyword)) {

	    read_xreal = true;
	    last_real = true;

	} else if (!strcmp("system_time", keyword)) {

	    // Check input format before reading.

	    if (!last_real) read_xreal = false;

	    if (read_xreal) {
		//cerr << "hdyn::scan_dyn_story: input "
		//     << "time data type is xreal"
		//     << endl; 
		system_time = get_xreal_from_input_line(input_line);
	    } else {
		real tmp;
		cerr << "hdyn::scan_dyn_story: input "
		     << "time data type is real"
		     << endl; 
		sscanf(input_line, "%*s%*s%lf", &tmp);
		system_time = tmp;
	    }
	    // PRC(system_time); xprint(system_time);

	} else {

	    last_real = false;

	    if (!strcmp("t", keyword)) {

		if (read_xreal)
		    time = get_xreal_from_input_line(input_line);
		else {
		    real tmp;
		    sscanf(input_line, "%*s%*s%lf", &tmp);
		    time = tmp;
		}

	    } else if (!strcmp("dt", keyword))
		sscanf(input_line, "%*s%*s%lf", &timestep);
	    else if (!strcmp("m", keyword))
		sscanf(input_line, "%*s%*s%lf", &mass);
	    else if (!strcmp("r", keyword))
		set_vector_from_input_line(pos, input_line);
	    else if (!strcmp("v", keyword)) {
		set_vector_from_input_line(vel, input_line);
		posvel = pos*vel;
	    } else if (!strcmp("a", keyword))
		set_vector_from_input_line(acc, input_line);
	    else if (!strcmp("pot", keyword))
		sscanf(input_line, "%*s%*s%lf", &pot);
	    else if (!strcmp("R_eff", keyword))
		sscanf(input_line, "%*s%*s%lf", &radius);
	    else if (!strcmp("steps", keyword))
		sscanf(input_line, "%*s%*s%lf", &steps);
	    else if (!strcmp("dir_f", keyword))
		sscanf(input_line, "%*s%*s%lf", &direct_force);
	    else if (!strcmp("indir_f", keyword))
		sscanf(input_line, "%*s%*s%lf", &indirect_force);
	    else if (!strcmp("dt_u", keyword))
		sscanf(input_line, "%*s%*s%lf", &unperturbed_timestep);
	    else if (!strcmp("full_u", keyword))
		sscanf(input_line, "%*s%*s%d", &fully_unperturbed);
	    else if (!strcmp("slow_kappa", keyword)) {




//////// Careful!!!  Are tree relationships established yet???




		// Component of a slow binary.  The information needed to
		// construct the slow structure is attached to the elder
		// sister only.

		int k;
		sscanf(input_line, "%*s%*s%d", &k);

		if (k > 1) {

		    if (!elder_sister) {

			// Create the slow structure.  Note that we won't need
			// to apply modifications to acc or the sister data.

			slow = new slow_binary(k);
			slow->set_dtau(timestep/k);

			// t_init, t_apo, and tau will be set in due course...

		    } else {

			// Elder sister should already have been created, and
			// contains all relevant data.

			slow = get_elder_sister()->slow;

		    }
		}

	    } else if (!strcmp("slow_t_init", keyword)) {

		if (!elder_sister && slow) {
		    real tmp;
		    sscanf(input_line, "%*s%*s%lf", &tmp);
		    slow->set_t_init(tmp);
		}

	    } else if (!strcmp("slow_t_apo", keyword)) {

		if (!elder_sister && slow) {
		    real tmp;
		    sscanf(input_line, "%*s%*s%lf", &tmp);
		    slow->set_t_apo(tmp);
		}

	    } else if (!strcmp("slow_tau", keyword)) {

		if (!elder_sister && slow) {
		    real tmp;
		    sscanf(input_line, "%*s%*s%lf", &tmp);
		    slow->set_tau(tmp);
		    slow->init_tau_pred();
		}

	    } else

		add_story_line(dyn_story, input_line);
	}
    }

    check_sanity_of_timestep(time, timestep);

    return s;
}

ostream & hdyn::print_dyn_story(ostream & s,
				bool print_xreal)	// default = true
{
    put_story_header(s, "Dynamics");

    if (!parent) {

	// See xreal notes in dyn_io.C...

#ifdef USE_XREAL
	if (print_xreal) {

	    put_real_number(s, "  real_system_time  =  ", (real)system_time);
	    put_real_number(s, "  system_time  =  ", system_time);

	} else

	    put_real_number(s, "  system_time  =  ", (real)system_time);
#else

	put_real_number(s, "  system_time  =  ", system_time);

#endif
    }

    if (print_xreal)
	put_real_number(s, "  t  =  ", time);		// OK for real or xreal
    else
	put_real_number(s, "  t  =  ", (real)time);

    put_real_number(s, "  dt =  ", timestep);
    put_real_number(s, "  m  =  ", mass);
    put_real_vector(s, "  r  =  ", pos);
    put_real_vector(s, "  v  =  ", vel);
    put_real_vector(s, "  a  =  ", acc);
    put_real_number(s, "  pot  =  ", pot);
    put_real_number(s, "  R_eff  =  ", radius);
    put_real_number(s, "  steps  =  ", steps);
    put_real_number(s, "  dir_f  =  ", direct_force);
    put_real_number(s, "  indir_f  =  ", indirect_force);

    if (kep) {

	// Component of an unperturbed binary -- may be fully or
	// partially unperturbed.  Store sufficient information for
	// restart.  Other (kepler) data can be recomputed from hdyn.

	put_real_number(s, "  dt_u  =  ", unperturbed_timestep);
	put_integer(s, "  full_u  =  ", fully_unperturbed);
    }

    if (slow) {

	// Component of a slow binary.  Value of kappa serves as a
	// flag for slow motion.  Attach additional information
	// needed for restart to elder component only.

	put_integer(s, "  slow_kappa  =  ", get_kappa());

	if (!elder_sister) {
	    put_real_number(s, "  slow_t_init  =  ", slow->get_t_init());
	    put_real_number(s, "  slow_t_apo  =  ", slow->get_t_apo());
	    put_real_number(s, "  slow_tau  =  ", slow->get_tau());
	}
    }

    if (dyn_story)
	put_story_contents(s, *dyn_story);

    put_story_footer(s, "Dynamics");

    return s;
}

typedef struct
{
    hdyn* b;
    real  d;
} bd_pair, *bd_pair_ptr;

local int compare(const void * pi, const void * pj)
{
    if (((bd_pair_ptr) pi)->d > ((bd_pair_ptr) pj)->d)
        return 1;
    else if (((bd_pair_ptr)pi)->d < ((bd_pair_ptr)pj)->d)
        return -1;
    else
        return 0;
}

void hdyn::print_perturber_list(ostream & s, char* pre)
{
    // NOTE: 'this' is a binary center of mass.

    s << pre << "perturber list of " << format_label() << flush;

    if (!valid_perturbers || perturber_list == NULL) {
	s << " is invalid" << endl;
	return;
    }

    int np = n_perturbers;

    if (np <= 0) {
	s << " is empty (no perturbers)" << endl;
	return;
    }

    // Sort the perturbers by distance (between top-level nodes).

    hdyn* top = get_top_level_node();

    // s << "creating list" << endl << flush;

    bd_pair_ptr bd_list = new bd_pair[np];
    for (int i = 0; i < np; i++) {
	bd_list[i].b = perturber_list[i];
	if (perturber_list[i] && perturber_list[i]->is_valid()) {
	    bd_list[i].d = square(perturber_list[i]
					  ->get_top_level_node()->pos
				  - top->pos);
	} else
	    bd_list[i].d = VERY_LARGE_NUMBER;
    }

    // s << "sorting list" << endl << flush;

    qsort((void *)bd_list, (size_t)np, sizeof(bd_pair), compare);

    s << "  (" << np << " perturbers, increasing distance):";
    s << endl << pre << "        ";
    for (int i = 0; i < np; i++) {
	if (bd_list[i].b)
	    bd_list[i].b->print_label(s);
	else
	    s << "(null)";
	if ((i+1)%10 == 0) {
	    if (i < np-1)
		s << endl << pre << "        ";
	} else
	    s << " ";
    }
    s << endl;

    delete [] bd_list;

    // s << "leaving print_perturber_list" << endl << flush;
}

void hdyn::find_print_perturber_list(ostream & s, char* pre)
{
    s << pre << "perturber list node for use by " << format_label() << " is ";

    hdyn* pnode = find_perturber_node();

    if (!pnode) {
	s << "NULL" << endl;
	return;
    } else
	s << pnode->format_label() << endl;

    pnode->print_perturber_list(s, pre);
}

#endif
