
// mkmass.C:	Add a (wrong) mass spectrum to a snapshot.
//
//		Steve McMillan, July 1996

#include "node.h"

#ifdef TOOLBOX

local void mkmass(node* b, real ml, real mu, real alpha, real m_total)
{
    real norm, m_sum = 0;

    // Normalizing factor.

    if (alpha == -2)
	norm = log(mu/ml);
    else
	norm = pow(mu/ml, 2+alpha) - 1;

    // Set all masses.

    for_all_daughters(node, b, bi) {
	if (alpha == -2)
	    bi->set_mass(ml*exp(norm*randinter(0,1)));
	else
	    bi->set_mass(ml*pow(norm*randinter(0,1) + 1, 1/(2+alpha)));
	m_sum += bi->get_mass();
    }
    b->set_mass(m_sum);

    // Optionally scale the total mass to the desired value.

    if (m_total > 0) {
	real m_factor = m_total/m_sum;
	for_all_daughters(node, b, bi)
	    bi->set_mass(m_factor*bi->get_mass());
	b->set_mass(m_total);
    }
}

void main(int argc, char ** argv)
{
    real m_lower = 1, m_upper = 1;		// Default = equal masses
    real exponent = -2.35;			// Salpeter mass function
    real m_total = -1;				// Don't rescale

    int random_seed = 0;
    char seedlog[64];

    extern char *poptarg;
    int c;
    char* param_string = "e:l:m:s:u:";

    while ((c = pgetopt(argc, argv, param_string)) != -1)
	switch(c) {

	    case 'E':
	    case 'e': exponent = atof(poptarg);
		      break;
	    case 'L':
	    case 'l': m_lower = atof(poptarg);
		      break;
	    case 'M':
	    case 'm': m_total = atof(poptarg);
		      break;
	    case 's': random_seed = atoi(poptarg);
		      break;
	    case 'U':
	    case 'u': m_upper = atof(poptarg);
		      break;
            case '?': params_to_usage(cerr, argv[0], param_string);
		      exit(1);
	}

    if (m_lower <= 0 || m_upper <= 0 || m_lower > m_upper)
	err_exit("mkmass: Illegal mass limits");

    node* b;
    b = get_node(cin);

    b->log_history(argc, argv);

    int actual_seed = srandinter(random_seed);

    sprintf(seedlog, "         random number generator seed = %d",actual_seed);
    b->log_comment(seedlog);

    mkmass(b, m_lower, m_upper, exponent, m_total);

    real initial_mass = getrq(b->get_dyn_story(), "initial_mass");

    if (initial_mass > -VERY_LARGE_NUMBER)
        putrq(b->get_dyn_story(), "initial_mass", m_total);

    put_node(cout, *b);

}
#endif
