//
//  mkdouble.C: construct a linked list of unit-mass nodes.
//
//	       Simon Portegies Zwart, July 1996
//

#include "dyn.h"
#include "double_star.h"
#include "single_star.h"
#include "main_sequence.h"
#include "dstar_to_dyn.h"


void add_secondary(dyn* original, real mass_ratio) {

    dyn* primary = new dyn;
    dyn* secondary = new dyn;

    // Add new links.

    original->set_oldest_daughter(primary);

    primary->set_parent(original);
    secondary->set_parent(original);

    primary->set_younger_sister(secondary);
    secondary->set_elder_sister(primary);

    // Set new masses.

    primary->set_mass(original->get_mass());
    secondary->set_mass(mass_ratio*original->get_mass());
    original->inc_mass(secondary->get_mass());

    // Naming convention:

    if (original->get_name() == NULL)
        if (original->get_index() >= 0) {
            char tmp[64];
            sprintf(tmp, "%d", original->get_index());
            original->set_name(tmp);
        }

    primary->set_name(original->get_name());
    secondary->set_name(original->get_name());
    strcat(primary->get_name(), "a");
    strcat(secondary->get_name(), "b");

   }

void mksecondary(dyn* b, real binary_fraction, real lower_limit) {

    // For now, use a flat distribution in secondary mass ratio.
    // Assume that the probability of a star being the primary of
    // a binary is independent of mass.

    real sum = 0;
    b->set_mass(0);

    for_all_daughters(dyn, b, bi) {
        sum += binary_fraction;
        if (sum >= 1) {
            sum -= 1;

            real mass_ratio = randinter(lower_limit, 1);        // Quick fix...
            add_secondary(bi, mass_ratio);

        }
        b->inc_mass(bi->get_mass());
    }
}

local void  evolve_binary(dyn* the_binary, real end_time) {

      real time = 0;   
      real dt=0;
      if (!the_binary->is_root())
         if (the_binary->get_parent()->is_root()) {
             cerr<<"binary "<<the_binary<<endl;
             do {
                dt = the_binary->get_starbase()->get_evolve_timestep();
                time += max(0., min(end_time-time, dt));
                cerr<<"dt, t= " << dt<<" " << time<<endl;
                the_binary->get_starbase()->evolve_element(time);
             }
	     while (time<end_time);
         }
   }
local  bool  evolve_the_stellar_system(dyn* b, real time) {

      for_all_nodes(dyn, b, bi) {
         cerr<<bi<< " is_root?: "<<bi->is_root()<<endl;
         cerr<<bi<< " is_parent?: "<<bi->is_parent()<<endl;
         if (!bi->is_root())
            if (bi->get_parent()->is_root()) {
                cerr<<"binary "<<bi<<endl;
                bi->get_starbase()->evolve_element(time);
            }

     }
      return true;
   }

void make_new_kepler_to_dyn(dyn* b, real m_total, real sma, real ecc,
			    int planar=0) {

     kepler *johannes = new kepler;

     real peri = 1; // Default value (unimportant unless ecc = 1).

     // For now, binary phase is random.

     real mean_anomaly = randinter(-PI, PI);

     make_standard_kepler(*johannes, 0, m_total, -0.5 * m_total / sma, ecc,
                          peri, mean_anomaly);
     set_random_orientation(*johannes, planar);

     b->set_kepler(johannes);
   }

void main(int argc, char ** argv) {

    bool m_flag = false;

    bool reandom_initialization = false;
    int  n = 1;
    int id=1;
    real m_tot=1,r_hm=1, t_hc=1;
    stellar_type type = Main_Sequence;
    binary_type bin_type = Detached;
    real binary_fraction = 1.0;
//		Initial conditions from Tutukov & Yunglesson 1993.
    real sma    = 138;
    real ecc    = 0.0;
    real m_prim = 13.1;
    real m_sec  =  9.8;
    real mass_ratio = 0.75;
    real lower_limit = 0.0;
    int random_seed = 0;
    char seedlog[64];
    real start_time = 0;
    real end_time   = 35;
    extern char *poptarg;
    int c;
    char* param_string = "a:e:M:m:n:q:s:T:";

    while ((c = pgetopt(argc, argv, param_string)) != -1)
	switch(c) {
            case 'a': sma = atof(poptarg);
                      break;
            case 'e': ecc = atof(poptarg);
                      break;
            case 'M': m_prim = atof(poptarg);
                      break;
            case 'm': m_flag = true;
		      m_sec = atof(poptarg);
                      break;
	    case 'n': n = atoi(poptarg);
		      break;
            case 'q': mass_ratio = atof(poptarg);
                      break;
            case 'T': end_time = atof(poptarg);
                      break;
            case 's': random_seed = atoi(poptarg);
                      break;
            case '?': params_to_usage(cerr, argv[0], param_string);
		      exit(1);
	}

    if (n <= 0) err_exit("mkdyn: N > 0 required!");
    int actual_seed = srandinter(random_seed);
    sprintf(seedlog, "         random number generator seed = %d",actual_seed);

    if (m_flag) 
       mass_ratio = m_sec / m_prim;
    else
       m_sec = mass_ratio*m_prim;
    m_tot = m_prim+m_sec;

    // Create flat tree 
    dyn *root  = mkdyn(n);
//    node *root  = mknode(n);
    root->log_history(argc, argv);
    root->log_comment(seedlog);
    root->get_starbase()->set_stellar_evolution_scaling(m_tot, r_hm, t_hc);
//    node* the_binary = root->get_oldest_daughter();
    dyn* the_binary = root->get_oldest_daughter();

//		Add the stars and secondary.
    add_secondary(the_binary, mass_ratio);
    addstar(root, start_time, type);
cerr<<"secondary and primary created"<<endl;
    double_star* new_double = 
		 new_double_star(the_binary, sma, ecc, 
				 start_time, id, bin_type);
cerr<<"Double-star created"<<endl;
    real total_mass = root->get_starbase()
		     ->conv_m_star_to_dyn(new_double->get_total_mass());
    real semi = root->get_starbase()
                     ->conv_r_star_to_dyn(sma);
    make_new_kepler_to_dyn(the_binary, total_mass, semi, ecc);
    put_dyn(cout, *root);

//	Test pointer structure
    cerr<<"est pointer structure"<<endl;
    dyn *b = root->get_oldest_daughter();
    starbase *s = b->get_starbase();
    star *st     = (star*)b->get_starbase();
    cerr<<"binary+s:" << b<<" "<<s<<" " <<st<<endl;
    cerr<<"a e m"<<s->get_effective_radius() <<" "
               <<s->get_total_mass()<<endl;
    cerr<<"p, s"<< st->get_primary()<<" "<<st->get_secondary() <<endl;
    cerr<<"M R, m r"<<st->get_primary()->get_total_mass() << " " 
                <<st->get_primary()->get_effective_radius() << " "
                <<st->get_secondary()->get_total_mass() << " " 
                <<st->get_secondary()->get_effective_radius()<< endl;
    cerr<<"companion p, s: "<< 
         st->get_secondary()->get_companion()<<" " 
       <<st->get_primary()->get_companion()<<endl;
   cerr<<"companion p, s: "<<st->get_companion(st->get_secondary())<<" "
                           <<st->get_companion(st->get_primary())<< endl; 

    for(int i=0; i< 100; i++) 
       evolve_the_stellar_system(root, end_time*i/100.);
//    evolve_binary(the_binary, end_time);
//    if (evolve_the_stellar_system(root, end_time)) {
       // rearrange_the_binary_dynamics_tree(root);
       // rearrange the tree; merge coalesced binaries and
       // 		      ionize disrupted binaries.
cerr<<"prepare to change dynamical mass of stars"<<endl;
//	change_dynamical_mass_of_the_stars(root);
cerr<<"all masses are updated! "<<endl;
//		CHANGE THE VELOCITIES ACCORDINLY.
//    }
    put_dyn(cout, *root);
}
